@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2000 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -391,13 +391,13 @@ static void do_scheme_rep(Scheme_Env *env, FinishArgs *fa)
 
 #ifdef GRAPHICAL_REPL
   if (!fa->a->alternate_rep) {
-    a[0] = scheme_intern_symbol("mred/mred");
+    a[0] = scheme_intern_symbol("racket/gui/init");
     a[1] = scheme_intern_symbol("graphical-read-eval-print-loop");
     ending_newline = 0;
   } else
 #endif
     {
-      a[0] = scheme_intern_symbol("scheme/base");
+      a[0] = scheme_intern_symbol("racket/base");
       a[1] = scheme_intern_symbol("read-eval-print-loop");
     }
 
