@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2012 PLT Scheme Inc.
   Copyright (c) 1995-2001 Matthew Flatt
 
     This library is free software; you can redistribute it and/or
@@ -599,19 +599,12 @@ Scheme_Object *scheme_make_toplevel(mzshort depth, int position, int resolved, i
   return (Scheme_Object *)tl;
 }
 
-Scheme_Object *scheme_register_toplevel_in_prefix(Scheme_Object *var, Scheme_Comp_Env *env,
-						  Scheme_Compile_Info *rec, int drec,
-                                                  int imported)
+Scheme_Object *scheme_register_toplevel_in_comp_prefix(Scheme_Object *var, Comp_Prefix *cp,
+                                                       int imported, Scheme_Object *inline_variant)
 {
-  Comp_Prefix *cp = env->prefix;
   Scheme_Hash_Table *ht;
   Scheme_Object *o;
 
-  if (rec && rec[drec].dont_mark_local_use) {
-    /* Make up anything; it's going to be ignored. */
-    return scheme_make_toplevel(0, 0, 0, 0);
-  }
-
   ht = cp->toplevels;
   if (!ht) {
     ht = scheme_make_hash_table(SCHEME_hash_ptr);
@@ -631,12 +624,36 @@ Scheme_Object *scheme_register_toplevel_in_prefix(Scheme_Object *var, Scheme_Com
                                   : SCHEME_TOPLEVEL_READY))
                             : 0));
 
-  cp->num_toplevels++;
   scheme_hash_set(ht, var, o);
 
+  if (inline_variant) {
+    ht = cp->inline_variants;
+    if (!ht) {
+      ht = scheme_make_hash_table(SCHEME_hash_ptr);
+      cp->inline_variants = ht;
+    }
+    scheme_hash_set(ht, scheme_make_integer(cp->num_toplevels), inline_variant);
+  }
+  
+  cp->num_toplevels++;
+
   return o;
 }
 
+Scheme_Object *scheme_register_toplevel_in_prefix(Scheme_Object *var, Scheme_Comp_Env *env,
+						  Scheme_Compile_Info *rec, int drec,
+                                                  int imported, Scheme_Object *inline_variant)
+{
+  Comp_Prefix *cp = env->prefix;
+
+  if (rec && rec[drec].dont_mark_local_use) {
+    /* Make up anything; it's going to be ignored. */
+    return scheme_make_toplevel(0, 0, 0, 0);
+  }
+
+  return scheme_register_toplevel_in_comp_prefix(var, cp, imported, inline_variant);
+}
+
 void scheme_register_unbound_toplevel(Scheme_Comp_Env *env, Scheme_Object *id)
 {
   Comp_Prefix *cp = env->prefix;
@@ -982,7 +999,7 @@ Scheme_Object *scheme_tl_id_sym(Scheme_Env *env, Scheme_Object *id, Scheme_Objec
 /* The `env' argument can actually be a hash table. */
 {
   Scheme_Object *marks = NULL, *sym, *map, *l, *a, *amarks, *m, *best_match, *cm, *abdg;
-  int best_match_skipped, ms, one_mark;
+  int best_match_skipped, ms;
   Scheme_Hash_Table *marked_names, *temp_marked_names, *dest_marked_names;
 
   sym = SCHEME_STX_SYM(id);
@@ -1061,9 +1078,7 @@ Scheme_Object *scheme_tl_id_sym(Scheme_Env *env, Scheme_Object *id, Scheme_Objec
        Since the list is otherwise marshaled into .zo, etc.,
        simplify by extracting just the mark: */
     marks = SCHEME_CAR(marks);
-    one_mark = 1;
-  } else
-    one_mark = 0;
+  }
 
   if (!SCHEME_TRUEP(bdg))
     bdg = NULL;
@@ -1253,11 +1268,11 @@ int scheme_tl_id_is_sym_used(Scheme_Hash_Table *marked_names, Scheme_Object *sym
   return 0;
 }
 
-static Scheme_Object *make_uid()
+static Scheme_Object *make_uid(int in_rib)
 {
   char name[20];
 
-  sprintf(name, "env%d", env_uid_counter++);
+  sprintf(name, "%cnv%d", in_rib ? 'r' : 'e', env_uid_counter++);
   return scheme_make_symbol(name); /* uninterned! */
 }
 
@@ -1268,7 +1283,7 @@ Scheme_Object *scheme_env_frame_uid(Scheme_Comp_Env *env)
 
   if (!env->uid) {
     Scheme_Object *sym;
-    sym = make_uid();
+    sym = make_uid(env->flags & SCHEME_FOR_INTDEF);
     env->uid = sym;
   }
   return env->uid;
@@ -1314,7 +1329,7 @@ static void make_env_renames(Scheme_Comp_Env *env, int rcount, int rstart, int r
       else
 	uid = env->uids[rstart];
       if (!uid)
-	uid = make_uid();
+	uid = make_uid(env->flags & SCHEME_FOR_INTDEF);
     }
   }
   
@@ -1659,7 +1674,8 @@ Scheme_Object *
 scheme_lookup_binding(Scheme_Object *find_id, Scheme_Comp_Env *env, int flags,
 		      Scheme_Object *in_modidx,
 		      Scheme_Env **_menv, int *_protected,
-                      Scheme_Object **_lexical_binding_id)
+                      Scheme_Object **_lexical_binding_id,
+                      Scheme_Object **_inline_variant)
 {
   Scheme_Comp_Env *frame;
   int j = 0, p = 0, modpos, skip_stops = 0, module_self_reference = 0, is_constant;
@@ -1963,7 +1979,11 @@ scheme_lookup_binding(Scheme_Object *find_id, Scheme_Comp_Env *env, int flags,
       is_constant = 2;
     else if (SAME_OBJ(mod_constant, scheme_fixed_key))
       is_constant = 1;
-    else {
+    else if (SAME_TYPE(SCHEME_TYPE(mod_constant), scheme_inline_variant_type)) {
+      if (_inline_variant)
+        *_inline_variant = mod_constant;
+      is_constant = 2;
+    } else {
       if (flags & SCHEME_ELIM_CONST) 
         return mod_constant;
       is_constant = 2;
@@ -2112,7 +2132,6 @@ int *scheme_env_get_flags(Scheme_Comp_Env *frame, int start, int count)
 Scheme_Object *
 scheme_do_local_lift_expr(const char *who, int stx_pos, int argc, Scheme_Object *argv[])
 {
-  Scheme_Env *menv;
   Scheme_Comp_Env *env, *orig_env;
   Scheme_Object *id, *ids, *rev_ids, *local_mark, *expr, *data, *vec, *id_sym;
   Scheme_Lift_Capture_Proc cp;  
@@ -2181,8 +2200,6 @@ scheme_do_local_lift_expr(const char *who, int stx_pos, int argc, Scheme_Object
   cp = *(Scheme_Lift_Capture_Proc *)SCHEME_VEC_ELS(vec)[1];
   data = SCHEME_VEC_ELS(vec)[2];
 
-  menv = scheme_current_thread->current_local_menv;
-
   orig_expr = expr;
 
   expr = cp(data, &ids, expr, orig_env);
@@ -2350,7 +2367,8 @@ Scheme_Object *scheme_namespace_lookup_value(Scheme_Object *sym, Scheme_Env *gen
   init_compile_data((Scheme_Comp_Env *)&inlined_e);
   inlined_e.base.prefix = NULL;
 
-  v = scheme_lookup_binding(id, (Scheme_Comp_Env *)&inlined_e, SCHEME_RESOLVE_MODIDS, NULL, NULL, NULL, NULL);
+  v = scheme_lookup_binding(id, (Scheme_Comp_Env *)&inlined_e, SCHEME_RESOLVE_MODIDS, 
+                            NULL, NULL, NULL, NULL, NULL);
   if (v) {
     if (!SAME_TYPE(SCHEME_TYPE(v), scheme_variable_type)) {
       *_use_map = -1;
