@@ -1,4 +1,4 @@
- Scheme_Object* scheme_rtcall_siS_s(const char *who, int src_type, prim_siS_s f, Scheme_Object* g56, int g57, Scheme_Object** g58)
+ Scheme_Object* scheme_rtcall_siS_s(const char *who, int src_type, prim_siS_s f, Scheme_Object* g59, int g60, Scheme_Object** g61)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -13,9 +13,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g56;
-    future->arg_i1 = g57;
-    future->arg_S2 = g58;
+      future->arg_s0 = g59;
+    future->arg_i1 = g60;
+    future->arg_S2 = g61;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -24,7 +24,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_iSs_s(const char *who, int src_type, prim_iSs_s f, int g59, Scheme_Object** g60, Scheme_Object* g61)
+ Scheme_Object* scheme_rtcall_iSs_s(const char *who, int src_type, prim_iSs_s f, int g62, Scheme_Object** g63, Scheme_Object* g64)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -39,9 +39,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g59;
-    future->arg_S1 = g60;
-    future->arg_s2 = g61;
+      future->arg_i0 = g62;
+    future->arg_S1 = g63;
+    future->arg_s2 = g64;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -50,7 +50,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_s_s(const char *who, int src_type, prim_s_s f, Scheme_Object* g62)
+ Scheme_Object* scheme_rtcall_s_s(const char *who, int src_type, prim_s_s f, Scheme_Object* g65)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -65,8 +65,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g62;
-  send_special_result(future, g62);
+      future->arg_s0 = g65;
+  send_special_result(future, g65);
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
   retval = future->retval_s;
@@ -74,7 +74,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_n_s(const char *who, int src_type, prim_n_s f, Scheme_Native_Closure_Data* g63)
+ Scheme_Object* scheme_rtcall_n_s(const char *who, int src_type, prim_n_s f, Scheme_Native_Closure_Data* g66)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -89,7 +89,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_n0 = g63;
+      future->arg_n0 = g66;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -122,7 +122,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_ss_s(const char *who, int src_type, prim_ss_s f, Scheme_Object* g64, Scheme_Object* g65)
+ Scheme_Object* scheme_rtcall_ss_s(const char *who, int src_type, prim_ss_s f, Scheme_Object* g67, Scheme_Object* g68)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -137,8 +137,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g64;
-    future->arg_s1 = g65;
+      future->arg_s0 = g67;
+    future->arg_s1 = g68;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -147,7 +147,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_ssi_s(const char *who, int src_type, prim_ssi_s f, Scheme_Object* g66, Scheme_Object* g67, int g68)
+ Scheme_Object* scheme_rtcall_ssi_s(const char *who, int src_type, prim_ssi_s f, Scheme_Object* g69, Scheme_Object* g70, int g71)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -162,9 +162,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g66;
-    future->arg_s1 = g67;
-    future->arg_i2 = g68;
+      future->arg_s0 = g69;
+    future->arg_s1 = g70;
+    future->arg_i2 = g71;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -173,7 +173,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_tt_s(const char *who, int src_type, prim_tt_s f, const Scheme_Object* g69, const Scheme_Object* g70)
+ Scheme_Object* scheme_rtcall_tt_s(const char *who, int src_type, prim_tt_s f, const Scheme_Object* g72, const Scheme_Object* g73)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -188,8 +188,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_t0 = g69;
-    future->arg_t1 = g70;
+      future->arg_t0 = g72;
+    future->arg_t1 = g73;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -198,7 +198,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- MZ_MARK_STACK_TYPE scheme_rtcall_ss_m(const char *who, int src_type, prim_ss_m f, Scheme_Object* g71, Scheme_Object* g72)
+ MZ_MARK_STACK_TYPE scheme_rtcall_ss_m(const char *who, int src_type, prim_ss_m f, Scheme_Object* g74, Scheme_Object* g75)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -213,8 +213,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g71;
-    future->arg_s1 = g72;
+      future->arg_s0 = g74;
+    future->arg_s1 = g75;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -223,7 +223,7 @@
   
   return retval;
 }
- Scheme_Object* scheme_rtcall_Sl_s(const char *who, int src_type, prim_Sl_s f, Scheme_Object** g73, intptr_t g74)
+ Scheme_Object* scheme_rtcall_Sl_s(const char *who, int src_type, prim_Sl_s f, Scheme_Object** g76, intptr_t g77)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -238,8 +238,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_S0 = g73;
-    future->arg_l1 = g74;
+      future->arg_S0 = g76;
+    future->arg_l1 = g77;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -248,7 +248,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_l_s(const char *who, int src_type, prim_l_s f, intptr_t g75)
+ Scheme_Object* scheme_rtcall_l_s(const char *who, int src_type, prim_l_s f, intptr_t g78)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -263,7 +263,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_l0 = g75;
+      future->arg_l0 = g78;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -272,7 +272,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_bsi_v(const char *who, int src_type, prim_bsi_v f, Scheme_Bucket* g76, Scheme_Object* g77, int g78)
+ void scheme_rtcall_bsi_v(const char *who, int src_type, prim_bsi_v f, Scheme_Bucket* g79, Scheme_Object* g80, int g81)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -287,9 +287,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_b0 = g76;
-    future->arg_s1 = g77;
-    future->arg_i2 = g78;
+      future->arg_b0 = g79;
+    future->arg_s1 = g80;
+    future->arg_i2 = g81;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -298,7 +298,7 @@
   
   
 }
- void scheme_rtcall_iiS_v(const char *who, int src_type, prim_iiS_v f, int g79, int g80, Scheme_Object** g81)
+ void scheme_rtcall_iiS_v(const char *who, int src_type, prim_iiS_v f, int g82, int g83, Scheme_Object** g84)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -313,9 +313,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g79;
-    future->arg_i1 = g80;
-    future->arg_S2 = g81;
+      future->arg_i0 = g82;
+    future->arg_i1 = g83;
+    future->arg_S2 = g84;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -324,7 +324,7 @@
   
   
 }
- void scheme_rtcall_ss_v(const char *who, int src_type, prim_ss_v f, Scheme_Object* g82, Scheme_Object* g83)
+ void scheme_rtcall_ss_v(const char *who, int src_type, prim_ss_v f, Scheme_Object* g85, Scheme_Object* g86)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -339,8 +339,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g82;
-    future->arg_s1 = g83;
+      future->arg_s0 = g85;
+    future->arg_s1 = g86;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -349,7 +349,7 @@
   
   
 }
- void scheme_rtcall_b_v(const char *who, int src_type, prim_b_v f, Scheme_Bucket* g84)
+ void scheme_rtcall_b_v(const char *who, int src_type, prim_b_v f, Scheme_Bucket* g87)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -364,7 +364,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_b0 = g84;
+      future->arg_b0 = g87;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -373,7 +373,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_sl_s(const char *who, int src_type, prim_sl_s f, Scheme_Object* g85, intptr_t g86)
+ Scheme_Object* scheme_rtcall_sl_s(const char *who, int src_type, prim_sl_s f, Scheme_Object* g88, intptr_t g89)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -388,8 +388,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g85;
-    future->arg_l1 = g86;
+      future->arg_s0 = g88;
+    future->arg_l1 = g89;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -398,7 +398,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_iS_s(const char *who, int src_type, prim_iS_s f, int g87, Scheme_Object** g88)
+ Scheme_Object* scheme_rtcall_iS_s(const char *who, int src_type, prim_iS_s f, int g90, Scheme_Object** g91)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -413,8 +413,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g87;
-    future->arg_S1 = g88;
+      future->arg_i0 = g90;
+    future->arg_S1 = g91;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -423,7 +423,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- Scheme_Object* scheme_rtcall_S_s(const char *who, int src_type, prim_S_s f, Scheme_Object** g89)
+ Scheme_Object* scheme_rtcall_S_s(const char *who, int src_type, prim_S_s f, Scheme_Object** g92)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -438,7 +438,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_S0 = g89;
+      future->arg_S0 = g92;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -447,7 +447,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_s_v(const char *who, int src_type, prim_s_v f, Scheme_Object* g90)
+ void scheme_rtcall_s_v(const char *who, int src_type, prim_s_v f, Scheme_Object* g93)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -462,8 +462,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g90;
-  send_special_result(future, g90);
+      future->arg_s0 = g93;
+  send_special_result(future, g93);
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
   
@@ -471,7 +471,7 @@
   
   
 }
- Scheme_Object* scheme_rtcall_iSi_s(const char *who, int src_type, prim_iSi_s f, int g91, Scheme_Object** g92, int g93)
+ Scheme_Object* scheme_rtcall_iSi_s(const char *who, int src_type, prim_iSi_s f, int g94, Scheme_Object** g95, int g96)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -486,9 +486,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g91;
-    future->arg_S1 = g92;
-    future->arg_i2 = g93;
+      future->arg_i0 = g94;
+    future->arg_S1 = g95;
+    future->arg_i2 = g96;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -497,7 +497,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_siS_v(const char *who, int src_type, prim_siS_v f, Scheme_Object* g94, int g95, Scheme_Object** g96)
+ void scheme_rtcall_siS_v(const char *who, int src_type, prim_siS_v f, Scheme_Object* g97, int g98, Scheme_Object** g99)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -512,9 +512,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g94;
-    future->arg_i1 = g95;
-    future->arg_S2 = g96;
+      future->arg_s0 = g97;
+    future->arg_i1 = g98;
+    future->arg_S2 = g99;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -523,7 +523,7 @@
   
   
 }
- void* scheme_rtcall_z_p(const char *who, int src_type, prim_z_p f, size_t g97)
+ void* scheme_rtcall_z_p(const char *who, int src_type, prim_z_p f, size_t g100)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -538,7 +538,7 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_z0 = g97;
+      future->arg_z0 = g100;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -547,7 +547,7 @@
   
   return retval;
 }
- Scheme_Object* scheme_rtcall_si_s(const char *who, int src_type, prim_si_s f, Scheme_Object* g98, int g99)
+ Scheme_Object* scheme_rtcall_si_s(const char *who, int src_type, prim_si_s f, Scheme_Object* g101, int g102)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -562,8 +562,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g98;
-    future->arg_i1 = g99;
+      future->arg_s0 = g101;
+    future->arg_i1 = g102;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -572,7 +572,7 @@
   receive_special_result(future, retval, 1);
   return retval;
 }
- void scheme_rtcall_sis_v(const char *who, int src_type, prim_sis_v f, Scheme_Object* g100, int g101, Scheme_Object* g102)
+ void scheme_rtcall_sis_v(const char *who, int src_type, prim_sis_v f, Scheme_Object* g103, int g104, Scheme_Object* g105)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -587,9 +587,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g100;
-    future->arg_i1 = g101;
-    future->arg_s2 = g102;
+      future->arg_s0 = g103;
+    future->arg_i1 = g104;
+    future->arg_s2 = g105;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -598,7 +598,7 @@
   
   
 }
- int scheme_rtcall_ss_i(const char *who, int src_type, prim_ss_i f, Scheme_Object* g103, Scheme_Object* g104)
+ int scheme_rtcall_ss_i(const char *who, int src_type, prim_ss_i f, Scheme_Object* g106, Scheme_Object* g107)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -613,8 +613,8 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_s0 = g103;
-    future->arg_s1 = g104;
+      future->arg_s0 = g106;
+    future->arg_s1 = g107;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -623,7 +623,7 @@
   
   return retval;
 }
- void scheme_rtcall_iSp_v(const char *who, int src_type, prim_iSp_v f, int g105, Scheme_Object** g106, void* g107)
+ void scheme_rtcall_iSp_v(const char *who, int src_type, prim_iSp_v f, int g108, Scheme_Object** g109, void* g110)
    XFORM_SKIP_PROC
 {
   Scheme_Future_Thread_State *fts = scheme_future_thread_state;
@@ -638,9 +638,9 @@
   future->time_of_request = tm;
   future->source_of_request = who;
   future->source_type = src_type;
-      future->arg_i0 = g105;
-    future->arg_S1 = g106;
-    future->arg_p2 = g107;
+      future->arg_i0 = g108;
+    future->arg_S1 = g109;
+    future->arg_p2 = g110;
   
   future_do_runtimecall(fts, (void*)f, 0, 1);
   future = fts->thread->current_ft;
@@ -648,4 +648,30 @@
   
   
   
+}
+ Scheme_Object* scheme_rtcall_sss_s(const char *who, int src_type, prim_sss_s f, Scheme_Object* g111, Scheme_Object* g112, Scheme_Object* g113)
+   XFORM_SKIP_PROC
+{
+  Scheme_Future_Thread_State *fts = scheme_future_thread_state;
+  future_t *future;
+  double tm;
+  Scheme_Object* retval;
+
+  future = fts->thread->current_ft;
+  future->prim_protocol = SIG_sss_s;
+  future->prim_func = f;
+  tm = get_future_timestamp();
+  future->time_of_request = tm;
+  future->source_of_request = who;
+  future->source_type = src_type;
+      future->arg_s0 = g111;
+    future->arg_s1 = g112;
+    future->arg_s2 = g113;
+  
+  future_do_runtimecall(fts, (void*)f, 0, 1);
+  future = fts->thread->current_ft;
+  retval = future->retval_s;
+  future->retval_s = 0;
+  receive_special_result(future, retval, 1);
+  return retval;
 }
